@@ -8,13 +8,6 @@ from .logger import plog
 
 ############ BandGuard Options #################
 
-# Kill a circuit if more than this many received cells are considered
-# invalid by Tor. This prevents an adversary from inserting cells
-# that are silently dropped into a circuit, to use as a timing side
-# channel.... To prevent dropmark attacks, a limit of 0 cells is used
-# until the first application data appears on a circuit.
-CIRC_MAX_DROPPED_CELLS = 30
-
 # Kill a circuit if this many read+write bytes have been exceeded.
 # Very loud application circuits could be used to introduce timing
 # side channels.
@@ -45,14 +38,6 @@ _CELL_PAYLOAD_SIZE = 509
 _RELAY_HEADER_SIZE = 11
 _RELAY_PAYLOAD_SIZE = _CELL_PAYLOAD_SIZE - _RELAY_HEADER_SIZE
 _CELL_DATA_RATE = (float(_RELAY_PAYLOAD_SIZE)/_CELL_PAYLOAD_SIZE)
-_RELAY_CELL_RATE = (float(_CELL_PAYLOAD_SIZE)/_RELAY_PAYLOAD_SIZE)
-
-# Constants from connection_edge_consider_sending_sendme(). These govern the
-# max SENDMEs can be expected to be in-flight (we use circ window because
-# there can be multiple streams on one circ).
-_STREAM_SENDME_INCREMENT = 50
-_STREAM_SENDME_WINDOW = 500
-_CIRC_SENDME_WINDOW = 1000
 
 _SECS_PER_HOUR = 60*60
 _BYTES_PER_KB = 1024
@@ -63,23 +48,6 @@ _BYTES_PER_MB = 1024*_BYTES_PER_KB
 # give it until the next couple in case there is a scheduled events hiccup
 _MAX_CIRC_DESTROY_LAG_SECS = 2
 
-# At least 150 bytes must be "delievered" to the application before
-# we allow any drops. This helps protect against DropMark even if
-# CIRC_MAX_DROPPED_CELLS is set.
-# WARN before this, NOTICE after
-_MIN_BYTES_UNTIL_DROPS = 150
-
-# Without #25573, optimistic data can cause us to send a bunch of
-# begins with optimistic data, and the service could send us
-# a whole stream window full of cells in response.
-_MAX_PATH_BIAS_CELLS_CLIENT = _STREAM_SENDME_WINDOW
-
-# Without #25573, the service situation is better. Path bias
-# exempts most service side circs because the rend is adversary
-# chosen, and the ones it doesn't only allow 1 cell through, then
-# the probe.
-_MAX_PATH_BIAS_CELLS_SERVICE = 2
-
 class BwCircuitStat:
   def __init__(self, circ_id, is_hs):
     self.circ_id = circ_id
@@ -88,7 +56,6 @@ class BwCircuitStat:
     self.is_hsdir = 0
     self.in_use = 0
     self.built = 0
-    self.path_bias_cells = 0
     self.created_at = time.time()
     self.read_bytes = 0
     self.sent_bytes = 0
@@ -110,6 +77,7 @@ class BwGuardStat:
   def __init__(self, guard_fp):
     self.to_guard = guard_fp
     self.killed_conns = 0
+    self.killed_conn_at = 0
     self.killed_conn_pending = False
     self.conns_made = 0
     self.close_reasons = {} # key=reason val=count
@@ -121,12 +89,18 @@ class BandwidthStats:
     self.live_guard_conns = {} # key=connid val=BwGuardStat
     self.guards = {} # key=guardfp val=BwGuardStat
     self.circs_destroyed_total = 0
-    self.no_conns_since = time.time()
+    self.no_conns_since = int(time.time())
     self.no_circs_since = None
+    self.network_down_since = None
     self.max_fake_id = -1
     self.disconnected_circs = False
     self.disconnected_conns = False
     self._orconn_init(controller)
+    self._network_liveness_init(controller)
+
+  def _network_liveness_init(self, controller):
+    if controller.get_info("network-liveness") != "up":
+      self.network_down_since = int(time.time())
 
   # Load in our current orconns. orconn-status does not
   # tell us IDs, so we have to fake it and keep track of fakes :/
@@ -203,9 +177,9 @@ class BandwidthStats:
     guardfp = event.path[0][0]
     if event.arrived_at - self.guards[guardfp].killed_conn_at \
         <= _MAX_CIRC_DESTROY_LAG_SECS:
-      self.guards[guardfp].killed_conn_at = None
+      self.guards[guardfp].killed_conn_at = 0
       self.guards[guardfp].killed_conns += 1
-      # FIXME: Limit to warn after?
+      # FIXME: Limit to warn after N killed connections?
       plog("NOTICE", "The connection to guard "+guardfp+" was closed with "+\
            "a live circuit.")
 
@@ -247,25 +221,24 @@ class BandwidthStats:
       return
 
     if event.id not in self.circs:
-      if event.hs_state or event.purpose[0:2] == "HS":
-        self.circs[event.id] = BwCircuitStat(event.id, 1)
+      self.circs[event.id] = BwCircuitStat(event.id,
+                        event.hs_state or event.purpose[0:2] == "HS")
 
-        # Handle direct build purpose settings
-        if event.purpose[0:9] == "HS_CLIENT":
-          self.circs[event.id].is_service = 0
-        elif event.purpose[0:10] == "HS_SERVICE":
-          self.circs[event.id].is_service = 1
-        if event.purpose == "HS_CLIENT_HSDIR" or \
-           event.purpose == "HS_SERVICE_HSDIR":
-          self.circs[event.id].is_hsdir = 1
-        plog("DEBUG", "Added hs circ for "+event.raw_content())
+      # Handle direct build purpose settings
+      if event.purpose[0:9] == "HS_CLIENT":
+        self.circs[event.id].is_service = 0
+      elif event.purpose[0:10] == "HS_SERVICE":
+        self.circs[event.id].is_service = 1
+      if event.purpose == "HS_CLIENT_HSDIR" or \
+         event.purpose == "HS_SERVICE_HSDIR":
+        self.circs[event.id].is_hsdir = 1
+      plog("DEBUG", "Added circ for "+event.raw_content())
 
     # Consider all BUILT circs that have a specific HS purpose
     # to be "in_use".
     if event.status == stem.CircStatus.BUILT or \
        event.status == "GUARD_WAIT":
-      if event.purpose[0:2] == "HS":
-        self.circs[event.id].built = 1
+      self.circs[event.id].built = 1
 
       if self.disconnected_circs:
         disconnected_secs = event.arrived_at - self.no_circs_since
@@ -311,13 +284,6 @@ class BandwidthStats:
         self.circs[event.id].guard_fp = event.path[0][0]
         plog("DEBUG", "Circ "+event.id+" now in-use. %d delivered bytes.",
              self.circs[event.id].delivered_read_bytes)
-      # XXX: We need to give path bias circs one extra cell during
-      # the DropMark check until #25573 is merged :/
-      if event.purpose == "PATH_BIAS_TESTING":
-        if self.circs[event.id].is_service:
-          self.circs[event.id].path_bias_cells = _MAX_PATH_BIAS_CELLS_SERVICE
-        else:
-          self.circs[event.id].path_bias_cells = _MAX_PATH_BIAS_CELLS_CLIENT
 
     plog("DEBUG", event.raw_content())
 
@@ -374,10 +340,23 @@ class BandwidthStats:
          self.any_circuits_pending():
         if not self.disconnected_circs or \
           disconnected_secs % CIRC_MAX_DISCONNECTED_SECS == 0:
-          plog("WARN", "Tor has been failing all circuits for %d seconds!"
-               % disconnected_secs)
+          if self.network_down_since:
+            plog("WARN", "Tor has been disconnected for %d seconds, "+\
+                 "and failing all circuits for %d seconds!",
+                  now - self.network_down_since,
+                  disconnected_secs)
+          else:
+            plog("NOTICE", "Tor has been failing all circuits for %d seconds!"
+                 % disconnected_secs)
         self.disconnected_circs = True
 
+  def network_liveness_event(self, event):
+    plog("DEBUG", event.raw_content())
+    if event.status == "UP":
+      self.network_down_since = None
+    elif event.status == "DOWN":
+      self.network_down_since = event.arrived_at
+
   def check_circ_ages(self, now):
     if CIRC_MAX_AGE_HOURS <= 0:
       return
@@ -402,26 +381,10 @@ class BandwidthStats:
     self.check_circ_ages(now)
 
   def check_circuit_limits(self, circ):
-    if not circ.is_hs: return
-
-    # DropMark hack. No dropped cells before app data.
-    if circ.delivered_read_bytes < _MIN_BYTES_UNTIL_DROPS:
-      # XXX: Until #25573 is merged, PATH_BIAS circs need a free cell
-      if circ.dropped_read_cells() > circ.path_bias_cells:
-        plog("NOTICE",
-             "Possible DropMark attack? Got an early dropped cell "+\
-             "before application data on circ %s. Closing circ.",
-             circ.circ_id)
-        control.try_close_circuit(self.controller, circ.circ_id)
-    elif circ.dropped_read_cells() > CIRC_MAX_DROPPED_CELLS:
-      # XXX: Until Tor ticket #25573 is merged, this must be notice.
-      loglevel = "NOTICE"
-      self.limit_exceeded(loglevel, "CIRC_MAX_DROPPED_CELLS",
-                          circ.circ_id,
-                          circ.dropped_read_cells(),
-                          CIRC_MAX_DROPPED_CELLS,
-                          "Total bytes: "+str(circ.read_bytes)+\
-                          "; All dropped: "+str(circ.dropped_read_cells()))
+    # Now that all supported tors have #25573, any dropped cell is bad.
+    if circ.dropped_read_cells() > 0:
+      plog("WARN",
+           "Possible attack! Got a dropped cell on circ %s.", circ.circ_id)
       control.try_close_circuit(self.controller, circ.circ_id)
     if CIRC_MAX_MEGABYTES > 0 and \
        circ.total_bytes() > CIRC_MAX_MEGABYTES*_BYTES_PER_MB:
