@@ -7,7 +7,6 @@ from vanguards.bandguards import BandwidthStats
 from vanguards.bandguards import CIRC_MAX_HSDESC_KILOBYTES
 from vanguards.bandguards import CIRC_MAX_MEGABYTES
 from vanguards.bandguards import CIRC_MAX_AGE_HOURS
-from vanguards.bandguards import CIRC_MAX_DROPPED_CELLS
 from vanguards.bandguards import CIRC_MAX_DISCONNECTED_SECS
 from vanguards.bandguards import CONN_MAX_DISCONNECTED_SECS
 
@@ -16,9 +15,6 @@ from vanguards.bandguards import _CELL_DATA_RATE
 from vanguards.bandguards import _SECS_PER_HOUR
 from vanguards.bandguards import _BYTES_PER_KB
 from vanguards.bandguards import _BYTES_PER_MB
-from vanguards.bandguards import _MIN_BYTES_UNTIL_DROPS
-from vanguards.bandguards import _MAX_PATH_BIAS_CELLS_CLIENT
-from vanguards.bandguards import _MAX_PATH_BIAS_CELLS_SERVICE
 
 import vanguards.logger
 
@@ -49,11 +45,17 @@ class MockController:
       return "$3E53D3979DB07EFD736661C934A1DED14127B684~Unnamed CONNECTED\n"+\
              "$3E53D3979DB07EFD736661C934A1DED14127B684~Unnamed LAUNCHED\n"+\
              "$3E53D3979DB07EFD736661C934A1DED14127B684~Unnamed CONNECTED"
+    if key == "network-liveness":
+      return "down"
 
 class MockEvent:
   def __init__(self, arrived_at):
     self.arrived_at = arrived_at
 
+def network_liveness_event(status):
+  s= "650 NETWORK_LIVENESS "+status+"\r\n"
+  return ControlMessage.from_str(s, "EVENT")
+
 def orconn_event(conn_id, guard, status):
   s= "650 ORCONN "+guard+" "+status+" ID="+str(conn_id)+"\r\n"
   return ControlMessage.from_str(s, "EVENT")
@@ -148,7 +150,6 @@ def check_dropped_bytes(state, controller, circ_id,
 
 # Test plan:
 def test_bwstats():
-  global CIRC_MAX_DROPPED_CELLS
   global CIRC_MAX_MEGABYTES
   controller = MockController()
   state = BandwidthStats(controller)
@@ -259,18 +260,13 @@ def test_bwstats():
   check_dropped_bytes(state, controller, circ_id, 0, 1)
   assert controller.closed_circ == str(circ_id)
 
-  # Test that 1 dropped cell is allowed on pathbias
+  # Test that 0 dropped cells are allowed on pathbias
   circ_id += 1
   controller.closed_circ = None
   state.circ_event(built_circ(circ_id, "HS_SERVICE_REND"))
   state.circ_minor_event(purpose_changed_circ(circ_id,
                                              "HS_SERVICE_REND",
                                              "PATH_BIAS_TESTING"))
-  path_bias_cells = 0
-  while path_bias_cells < _MAX_PATH_BIAS_CELLS_SERVICE:
-    check_dropped_bytes(state, controller, circ_id, 0, 1)
-    assert controller.closed_circ == None
-    path_bias_cells += 1
   check_dropped_bytes(state, controller, circ_id, 0, 1)
   assert controller.closed_circ == str(circ_id)
 
@@ -281,11 +277,6 @@ def test_bwstats():
   state.circ_minor_event(purpose_changed_circ(circ_id,
                                              "HS_CLIENT_REND",
                                              "PATH_BIAS_TESTING"))
-  path_bias_cells = 0
-  while path_bias_cells < _MAX_PATH_BIAS_CELLS_CLIENT:
-    check_dropped_bytes(state, controller, circ_id, 0, 1)
-    assert controller.closed_circ == None
-    path_bias_cells += 1
   check_dropped_bytes(state, controller, circ_id, 0, 1)
   assert controller.closed_circ == str(circ_id)
 
@@ -296,19 +287,38 @@ def test_bwstats():
   check_dropped_bytes(state, controller, circ_id, 0, 1)
   assert controller.closed_circ == str(circ_id)
 
-  # Test that after app data, up to CIRC_MAX_DROPPED_CELLS
-  # allowed, and then we close.
+  # Test that after app data, no dropped cells allowed,
+  # and then we close.
   circ_id += 1
   controller.closed_circ = None
   state.circ_event(built_circ(circ_id, "HS_VANGUARDS"))
   check_dropped_bytes(state, controller, circ_id,
-                      1000, CIRC_MAX_DROPPED_CELLS+1)
+                      1000, 1)
   assert controller.closed_circ == str(circ_id)
 
-  # - Non-HS circs ignored:
+  # Test Non-HS circs closed with dropped cells.
   circ_id += 1
+  controller.closed_circ = None
   state.circ_event(built_general_circ(circ_id))
-  assert str(circ_id) not in state.circs
+  check_dropped_bytes(state, controller, circ_id, 1000, 1)
+  assert controller.closed_circ == str(circ_id)
+
+  # Test that with #25573, no dropped cell is allowed
+  circ_id += 1
+  controller.closed_circ = None
+  state.tor_has_25573 = True
+  state.circ_event(built_circ(circ_id, "HS_VANGUARDS"))
+  check_dropped_bytes(state, controller, circ_id, 1000, 1)
+  assert controller.closed_circ == str(circ_id)
+
+  # Test that with #25573, no dropped cell is allowed
+  circ_id += 1
+  controller.closed_circ = None
+  state.tor_has_25573 = True
+  state.circ_event(built_general_circ(circ_id))
+  check_dropped_bytes(state, controller, circ_id, 1000, 1)
+  assert controller.closed_circ == str(circ_id)
+
 
 def test_connguard():
   controller = MockController()
@@ -323,23 +333,27 @@ def test_connguard():
   assert "1" not in state.live_guard_conns
   assert state.live_guard_conns["2"].to_guard == "3E53D3979DB07EFD736661C934A1DED14127B684"
 
-  # Test HS_INTRO close
+  # Test HS_INTRO close with an extra circ on it.
   assert state.circs_destroyed_total == 0
   state.orconn_event(
          orconn_event(11,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CONNECTED"))
   state.circ_event(built_circ(23, "HS_SERVICE_INTRO",
                               "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
+  state.circ_event(built_circ(24, "HS_SERVICE_INTRO",
+                              "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
   state.orconn_event(
          orconn_event(11,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CLOSED"))
-  state.circ_event(destroyed_circ(23,
+  state.circ_event(destroyed_circ(24,
                     "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
+  state.circ_event(destroyed_circ(23,
+                   "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 1
-  assert state.circs_destroyed_total == 1
+  assert state.circs_destroyed_total == 2
 
   # Test cannibalized close
-  assert state.circs_destroyed_total == 1
+  assert state.circs_destroyed_total == 2
   state.orconn_event(
          orconn_event(12,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CONNECTED"))
@@ -353,10 +367,10 @@ def test_connguard():
   state.circ_event(destroyed_circ(24,
                     "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 2
-  assert state.circs_destroyed_total == 2
+  assert state.circs_destroyed_total == 3
 
   # Test cannibalized close on pre-existing conn
-  assert state.circs_destroyed_total == 2
+  assert state.circs_destroyed_total == 3
   state.circ_event(built_circ(2323, "HS_VANGUARDS",
                               "$3E53D3979DB07EFD736661C934A1DED14127B684~Unnamed"))
   state.circ_minor_event(purpose_changed_circ(2323, "HS_VANGUARDS", "HS_SERVICE_REND",
@@ -371,11 +385,11 @@ def test_connguard():
                     "$3E53D3979DB07EFD736661C934A1DED14127B684~Unnamed"))
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 2
   assert state.guards["3E53D3979DB07EFD736661C934A1DED14127B684"].killed_conns == 1
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
 
   # Tests that should not trigger logs:
   #  * Test HS_VANGUARDS
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
   state.orconn_event(
          orconn_event(13,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CONNECTED"))
@@ -387,10 +401,10 @@ def test_connguard():
   state.circ_event(destroyed_circ(25,
                     "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 2
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
 
   #  * Test general close
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
   state.orconn_event(
          orconn_event(14,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CONNECTED"))
@@ -402,10 +416,10 @@ def test_connguard():
   state.circ_event(destroyed_circ(26,
                     "$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 2
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
 
   #  * Test late close of hs_intro
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
   state.orconn_event(
          orconn_event(15,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CONNECTED"))
@@ -419,10 +433,10 @@ def test_connguard():
   ev.arrived_at = time.time()+5
   state.circ_event(ev)
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 2
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
 
   #  * Different guard for hs_intro
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
   state.orconn_event(
          orconn_event(16,"$5416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed",
                       "CONNECTED"))
@@ -434,7 +448,7 @@ def test_connguard():
   state.circ_event(destroyed_circ(28,
                     "$6416F3E8F80101A133B1970495B04FDBD1C7446B~Unnamed"))
   assert state.guards["5416F3E8F80101A133B1970495B04FDBD1C7446B"].killed_conns == 2
-  assert state.circs_destroyed_total == 3
+  assert state.circs_destroyed_total == 4
 
   # Test orconn-status fake_id clearing, esp with launched
   assert len(state.live_guard_conns) == 1
@@ -515,7 +529,11 @@ def test_connguard():
   ev = failed_circ(31)
   ev.arrived_at = last_conn
   state.circ_event(ev)
+  ev = network_liveness_event("DOWN")
+  ev.arrived_at = last_conn
+  state.network_liveness_event(ev)
   assert state.no_circs_since
+  assert state.network_down_since
   ev.arrived_at = last_conn+CIRC_MAX_DISCONNECTED_SECS*2
   state.bw_event(ev)
   assert state.disconnected_circs == True
@@ -524,7 +542,11 @@ def test_connguard():
   ev = built_general_circ(32)
   ev.arrived_at = last_conn
   state.circ_event(ev)
+  ev = network_liveness_event("UP")
+  ev.arrived_at = last_conn
+  state.network_liveness_event(ev)
   assert state.no_circs_since == None
+  assert state.network_down_since == None
   state.bw_event(ev)
   assert state.disconnected_circs == False
 
